@@ -12,8 +12,7 @@
 **/
 
 
-#include <R.h>
-#include <Rmath.h>
+#include "spdep.h"
 
 
 static double distance(double x1, double y1, double x2, double y2){
